import requests
from bs4 import BeautifulSoup
import re

from . import BaseRepoParser

class CondaForgeRepoParser(BaseRepoParser):
    _base_url = 'https://anaconda.org/conda-forge/{pkg}'
    _license_regex = re.compile(r'License: ([a-zA-Z\-\_ 0-9]+)', re.MULTILINE)

    def _fetch_single(self, pkg: str):
        url = self._base_url.format(pkg=pkg)
        r = requests.get(url)

        try:
            soup = BeautifulSoup(r.text, features='html.parser')
            license = self.__query_license(soup)
            version = self.__query_version(soup)
        except:
            license = 'unknown'
            version = 'unknown'

        return {
            'name': pkg,
            'license': license,
            'version': version,
        }

    def __query_license(self, soup: BeautifulSoup):
        el = soup.find('li', {'title': 'License'})
        matches = self._license_regex.match(el.text.strip())
        return matches[1] if matches else 'unknown'

    def __query_version(self, soup: BeautifulSoup):
        el = soup.find('small', {'class': 'subheader'})
        return el.text.strip()





import requests

from . import BaseRepoParser

class PyPIRepoParser(BaseRepoParser):
    _base_url = 'https://pypi.org/pypi/{pkg}/json'

    def _fetch_single(self, pkg: str):
        url = self._base_url.format(pkg=pkg)
        r = requests.get(url)

        try:
            content = r.json()
            license = content['info']['license']
            version = content['info']['version']
        except:
            license = 'unknown'
            version = 'unknown'

        return {
            'name': pkg,
            'license': license,
            'version': version,
        }




from typing import Union, List, Dict
import re
import pandas as pd


class BaseRepoParser(object):
    _base_url: str = ""
    _line_regex = re.compile(r"^([A-Za-z0-9\-\_]+)")

    def _fetch_single(self, pkg: str):
        raise NotImplementedError()

    def _fetch_multiple(self, pkgs: List[str]):
        return [self._fetch_single(pkg) for pkg in pkgs]

    def _fetch(self, pkgs: Union[str, List[str]]):
        if type(pkgs) == str:
            return self._fetch_single(pkgs)
        else:
            return self._fetch_multiple(pkgs)

    def _match_input_line(self, line):
        return self._line_regex.match(line)

    def _read_from_file(self, filepath: str):
        with open(filepath, "r") as f:
            lines = f.read().split("\n")

        pkgs = []
        for line in lines:
            matches = self._match_input_line(line)
            if matches is None:
                continue
            pkgs.append(matches[1])
        return sorted(pkgs)

    def from_file(self, filepath: str):
        pkgs = self._read_from_file(filepath)
        pkgs_info = self._fetch(pkgs)
        return pkgs_info

    def as_csv(self, pkgs_info: List[Dict], separator="|"):
        df = pd.DataFrame.from_dict(pkgs_info)
        return df.to_csv(index=False, sep=separator)

    def as_markdown(self, pkgs_info: List[Dict]):
        df = pd.DataFrame.from_dict(pkgs_info)
        return df.to_markdown(index=False)


from argparse import ArgumentParser, Namespace
import pylicense.repos as repos


def dispatch(parser: ArgumentParser, cli_args: Namespace):
    if cli_args.repository == "pypi":
        r_parser = repos.PyPIRepoParser()
    elif cli_args.repository == "anaconda":
        r_parser = repos.AnacondaRepoParser()
    elif cli_args.repository == "conda-forge":
        r_parser = repos.CondaForgeRepoParser()

    pkgs_info = r_parser.from_file(cli_args.inputfile)
    if cli_args.output_format == "csv":
        print(r_parser.as_csv(pkgs_info))
    elif cli_args.output_format == "markdown":
        print(r_parser.as_markdown(pkgs_info))


import argparse


def configure_parser():
    parser = argparse.ArgumentParser(
        description="""
PyLicense is a CLI tool that queries license information from 
one of the following package repositories: PyPI, Anaconda or conda-forge.

It requires a file in the format of a standard requirements.txt file with one
package per line.
"""
    )

    parser.add_argument("-v", "--verbose", dest="verbosity", action="count", default=0)

    parser.add_argument(
        "inputfile", help="Input file in the form of a requirements.txt file"
    )
    parser.add_argument(
        "-r",
        "--repository",
        default="pypi",
        choices=["pypi", "anaconda", "conda-forge"],
        help="Repository that should be queried.",
    )
    parser.add_argument(
        "--output-format",
        default="markdown",
        choices=["csv", "markdown"],
        help="Output format of the resulting table",
    )

    return parser


import logging
import sys

import pylicense.constants as pc

def configure_logger() -> logging.Logger:
    logger = logging.getLogger(pc.DEFAULT_LOGGER_NAME)
    handler = logging.StreamHandler(sys.stdout)
    formatter = logging.Formatter(pc.LOG_FORMAT)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(logging.INFO)
    return logger


from pylicense.dispatcher import dispatch
from pylicense.parser import configure_parser
from pylicense.logger import configure_logger
import pylicense.constants as pc

def run():
    logger = configure_logger()
    parser = configure_parser()

    cli_args = parser.parse_args()
    log_level = pc.LOG_LEVELS[min(cli_args.verbosity, len(pc.LOG_LEVELS) - 1)]
    logger.setLevel(log_level)

    logger.debug(f"Parsed CLI arguments: {cli_args}")

    dispatch(parser, cli_args)


if __name__ == "__main__":
    run()


from typing import List
import logging

DEFAULT_LOGGER_NAME: str = 'pylicense'
LOG_FORMAT: str = logging.BASIC_FORMAT
LOG_LEVELS: List[int] = [logging.ERROR, logging.WARN, logging.INFO, logging.DEBUG]